﻿using System;
using System.Globalization;
using System.Web;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;

namespace RH
{
    /// <summary>
    /// by René Hézser
    /// http://www.hezser.de/blog
    /// </summary>
    class Helper
    {
        /// <summary>
        /// find a list by the title
        /// </summary>
        /// <param name="web"></param>
        /// <param name="listName"></param>
        /// <returns></returns>
        internal static SPList GetList(SPWeb web, string listName)
        {
            foreach (SPList list in web.Lists)
            {
                if (list.Title == listName)
                {
                    WriteULSVerbose(string.Format("List '{0}' found", listName));
                    return list;
                }
            }

            WriteULSVerbose(string.Format("List '{0}' not found", listName));
            return null;
        }

        /// <summary>
        /// Get a list by its Id
        /// </summary>
        /// <param name="web"></param>
        /// <param name="listId">Guid</param>
        /// <returns></returns>
        internal static SPList GetList(SPWeb web, Guid listId)
        {
            try
            {
                SPList list = web.Lists.GetList(listId, true);
                return list;
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in 'GetList' for Web '{0}' and ListId '{1}': {2}", web.Url, listId, ex));
                return null;
            }
        }

        /// <summary>
        /// checks if the feature is activated for the current spweb
        /// </summary>
        internal static bool FeatureActivated(SPWeb web, Guid featureId)
        {
            foreach (SPFeature feature in web.Features)
            {
                if (feature.DefinitionId == featureId)
                {
                    WriteULSVerbose(string.Format("Feature '{0}' is activated", feature.Definition.Name));
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// returns the current cultureinfo
        /// </summary>
        /// <returns></returns>
        internal static CultureInfo GetCultureInfo(SPWeb web)
        {
            CultureInfo newCultureInfo;
            try
            {
                var currentUser = GetCurrentUser(web);
                if (currentUser != null && currentUser.RegionalSettings != null && currentUser.RegionalSettings.LocaleId > 0)
                {
                    uint localeID = currentUser.RegionalSettings.LocaleId;
                    newCultureInfo = new CultureInfo((int)localeID);
                }
                else
                {
                    WriteULSVerbose("Current User is null (or its regionalSettings). Using web.RegionalSettings");
                    newCultureInfo = new CultureInfo((int)web.RegionalSettings.LocaleId);
                }
            }
            catch (Exception ex)
            {
                WriteULSError(string.Format("Error in GetCultureInfo for web '{0}': {1}", web.Url, ex));
                newCultureInfo = new CultureInfo(1033);
            }

            return newCultureInfo;
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Exception)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteULSError(string text)
        {
            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Exception, Guid.Empty, "RH.WikiWebpart.dll", "RH.WikiWebpart", "General", text);
                TraceProvider.UnregisterTraceProvider();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// write to the ULS Log (TraceSeverity = Verbose)
        /// </summary>
        /// <param name="text"></param>
        internal static void WriteULSVerbose(string text)
        {
            try
            {
                TraceProvider.RegisterTraceProvider();
                TraceProvider.WriteTrace(0, TraceProvider.TraceSeverity.Verbose, Guid.Empty, "RH.WikiWebpart.dll", "RH.WikiWebpart", "General", text);
                TraceProvider.UnregisterTraceProvider();
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// get the current user from the web. if it does not exist, add the user
        /// </summary>
        /// <param name="web"></param>
        /// <returns></returns>
        internal static SPUser GetCurrentUser(SPWeb web)
        {
            WriteULSVerbose(string.Format("Getting current user from web '{0}'", web.Url));
            if (web.CurrentUser != null)
            {
                WriteULSVerbose(string.Format("current user '{0}' found", web.CurrentUser.LoginName));
                return web.CurrentUser;
            }

            // ensure that the user is logged on
            WriteULSVerbose("Ensuring that the user is logged on");
            SPUtility.EnsureAuthentication();

            // ensure the user, if it does not exist
            using (var site = new SPSite(SPContext.Current.Site.Url, SPContext.Current.Site.SystemAccount.UserToken))
            using (SPWeb rootWeb = site.RootWeb)
            {
                bool allowUnsafeUpdate = rootWeb.AllowUnsafeUpdates;
                try
                {
                    rootWeb.AllowUnsafeUpdates = true;

                    // Get user identity name
                    string loginName = HttpContext.Current.User.Identity.Name;
                    WriteULSVerbose(string.Format("Ensuring user '{0}' on rootWeb '{1}'", loginName, rootWeb.Url));
                    SPUser currentUser = rootWeb.EnsureUser(loginName);

                    if (currentUser == null)
                        WriteULSError(string.Format("Could not get user '{0}' in web '{1}'", loginName, web.Url));
                    else
                        WriteULSVerbose(string.Format("Returning user '{0}' in web '{1}'", currentUser.LoginName, web.Url));

                    return currentUser;
                }
                finally
                {
                    rootWeb.AllowUnsafeUpdates = allowUnsafeUpdate;
                }
            }
        }

        /// <summary>
        /// get javascript "getTagFromIdentifierAndTitle"
        /// </summary>
        /// <returns></returns>
        internal static string JavaScriptGetTagFromIdentifierAndTitle()
        {
            string js = "function getTagFromIdentifierAndTitle(tagName, identifier, title)\r\n";
            js += "{\r\n";
            js += "    var len = identifier.length;\r\n";
            js += "    var tags = document.getElementsByTagName(tagName);\r\n";
            js += "    for (var i=0; i < tags.length; i++)\r\n";
            js += "    {\r\n";
            js += "        var tempString = tags[i].id;\r\n";
            js += "        if (tags[i].title == title && (identifier == \"\" || tempString.indexOf(identifier) == tempString.length - len))\r\n";
            js += "        {\r\n";
            js += "            return tags[i];\r\n";
            js += "        }\r\n";
            js += "    }\r\n";
            js += "    return null;\r\n";
            js += "}\r\n";

            js += "String.prototype.contains = function(t) { return this.indexOf(t) >= 0 ? true : false }\r\n";

            return js;
        }
    }
}
